@@ -20,7 +20,6 @@ import pyarrow.compute as pc
 import pyarrow.csv as pa_csv
 import pyarrow.parquet as pq
 from bs4 import BeautifulSoup
-from fastcore.parallel import parallel
 from rich import box
 from rich.columns import Columns
 from rich.console import Group
@@ -32,18 +31,21 @@ from rich.text import Text
 from edgar._markdown import text_to_markdown
 from edgar._party import Address
 from edgar.attachments import FilingHomepage, Attachment, Attachments, AttachmentServer
-from edgar.core import (log, display_size, sec_edgar,
-                        filter_by_date,
-                        filter_by_form,
-                        filter_by_cik,
-                        filter_by_exchange,
-                        filter_by_ticker,
-                        filter_by_accession_number,
+from edgar.filtering import (
+    filter_by_date,
+    filter_by_form,
+    filter_by_cik,
+    filter_by_exchange,
+    filter_by_ticker,
+    filter_by_accession_number
+)
+from edgar.dates import InvalidDateException
+from edgar.formatting import display_size
+from edgar.core import (log, sec_edgar,
                         listify,
                         cache_except_none,
                         is_start_of_quarter,
-                        has_html_content,
-                        InvalidDateException,
+                        is_probably_html,
                         IntString,
                         current_year_and_quarter,
                         Years,
@@ -53,7 +55,9 @@ from edgar.core import (log, display_size, sec_edgar,
                         quarters_in_year,
                         filing_date_to_year_quarters,
                         DataPager,
-                        PagingState)
+                        PagingState,
+                        parallel_thread_map)
+from edgar.formatting import accession_number_text
 from edgar.files.html import Document
 from edgar.files.html_documents import get_clean_html
 from edgar.files.htmltools import html_sections
@@ -63,13 +67,13 @@ from edgar.httprequests import download_file, download_text, download_text_betwe
 from edgar.httprequests import get_with_retry
 from edgar.reference import describe_form
 from edgar.reference.tickers import Exchange
-from edgar.reference.tickers import find_ticker
-from edgar.richtools import repr_rich, print_rich, rich_to_text
+from edgar.reference.tickers import find_ticker, find_ticker_safe
+from edgar.richtools import repr_rich, print_rich, rich_to_text, Docs
 from edgar.search import BM25Search, RegexSearch
 from edgar.sgml import FilingSGML, Reports, Statements, FilingHeader
 from edgar.storage import local_filing_path, is_using_local_storage
-from edgar.xbrl import XBRLData, XBRLInstance, get_xbrl_object
 from edgar.xmltools import child_text
+from edgar.xbrl import XBRL, XBRLFilingWithNoXbrlData
 
 """ Contain functionality for working with SEC filing indexes and filings
 
@@ -91,6 +95,8 @@ __all__ = [
     'CurrentFilings',
     'available_quarters',
     'get_current_filings',
+    'iter_current_filings_pages',
+    'get_all_current_filings',
     'get_by_accession_number',
     'filing_date_to_year_quarters'
 ]
@@ -108,6 +114,8 @@ form_index = "form"
 xbrl_index = "xbrl"
 company_index = "company"
 
+index_field_delimiter_re = re.compile(r" {2,}")
+
 max_concurrent_http_connections = 10
 
 accession_number_re = re.compile(r"\d{10}-\d{2}-\d{6}$")
@@ -147,6 +155,8 @@ def is_valid_date(date_str: str, date_format: str = "%Y-%m-%d") -> bool:
 
 
 def get_previous_quarter(year, quarter) -> Tuple[int, int]:
+    if not quarter:
+        _, quarter = current_year_and_quarter()
     # Given a year and quarter return the previous quarter
     if quarter == 1:
         return year - 1, 4
@@ -219,8 +229,9 @@ company_specs = FileSpecs(
      ]
 )
 
-FORM_INDEX_COLUMNS = ['form', 'company', 'cik', 'filing_date', 'accession_number']
-COMPANY_INDEX_COLUMNS = ['company', 'form', 'cik', 'filing_date', 'accession_number']
+FORM_INDEX_FORM_COLUMN = 0
+COMPANY_INDEX_FORM_COLUMN = -4
+INDEX_COLUMN_NAMES = ['form', 'company', 'cik', 'filing_date', 'accession_number']
 
 
 def read_fixed_width_index(index_text: str,
@@ -268,7 +279,7 @@ def read_fixed_width_index(index_text: str,
     )
 
 
-def read_index_file(index_text: str, columns: List[str] = FORM_INDEX_COLUMNS) -> pa.Table:
+def read_index_file(index_text: str, form_column: int = FORM_INDEX_FORM_COLUMN) -> pa.Table:
     """
     Read the index text using multiple spaces as delimiter
     """
@@ -283,14 +294,17 @@ def read_index_file(index_text: str, columns: List[str] = FORM_INDEX_COLUMNS) ->
     # Process data lines
     data_lines = lines[data_start:]
 
-    # Split each line by 2 or more spaces
-    rows = [line.split() for line in data_lines if line.strip()]
+    # Handle empty lines
+    if not data_lines:
+        return _empty_filing_index()
 
-    # Convert to arrays
-    forms = pa.array([line[:12].strip() for line in data_lines])  # The form might contain spaces like '1-A POS'
+    # The form and company name can both contain spaces the remaining fields cannot.
+    # It is assumed that the form will only contain runs of a single space (e.g. "1-A POS")
+    # so splitting on runs of 2 spaces or more will keep form names intact.
+    rows = [re.split(index_field_delimiter_re, line.strip()) for line in data_lines if line.strip()]
 
-    # Company names may have single spaces within them
-    companies = pa.array([' '.join(row[1:-3]) for row in rows])
+    # Form names are in a different column depending on the index type.
+    forms = pa.array([row[form_column] for row in rows]) 
 
     # CIKs are always the third-to-last field
     ciks = pa.array([int(row[-3]) for row in rows], type=pa.int32())
@@ -302,20 +316,27 @@ def read_index_file(index_text: str, columns: List[str] = FORM_INDEX_COLUMNS) ->
     # Accession numbers are in the file path
     accession_numbers = pa.array([row[-1][-24:-4] for row in rows])
 
+    # Company names may have runs of more than one space so anything which hasn't already
+    # been extracted is concatenated to form the company name.
+    if form_column == 0:
+        companies = pa.array([" ".join(row[1:-3]) for row in rows])
+    else:
+        companies = pa.array([" ".join(row[0:form_column]) for row in rows])
+
     return pa.Table.from_arrays(
         [forms, companies, ciks, dates, accession_numbers],
-        names=columns
+        names=INDEX_COLUMN_NAMES
     )
 
 
 def read_form_index_file(index_text: str) -> pa.Table:
     """Read the form index file"""
-    return read_index_file(index_text, columns=FORM_INDEX_COLUMNS)
+    return read_index_file(index_text, form_column=FORM_INDEX_FORM_COLUMN)
 
 
 def read_company_index_file(index_text: str) -> pa.Table:
     """Read the company index file"""
-    return read_index_file(index_text, columns=COMPANY_INDEX_COLUMNS)
+    return read_index_file(index_text, form_column=COMPANY_INDEX_FORM_COLUMN)
 
 
 def read_pipe_delimited_index(index_text: str) -> pa.Table:
@@ -373,8 +394,8 @@ def fetch_filing_index_at_url(url: str,
         index_table: pa.Table = read_pipe_delimited_index(str(index_text))
     else:
         # Read as a fixed width index file
-        columns = FORM_INDEX_COLUMNS if index == "form" else COMPANY_INDEX_COLUMNS
-        index_table: pa.Table = read_index_file(index_text, columns=columns)
+        form_column = FORM_INDEX_FORM_COLUMN if index == "form" else COMPANY_INDEX_FORM_COLUMN
+        index_table: pa.Table = read_index_file(index_text, form_column=form_column)
     return index_table
 
 
@@ -410,12 +431,10 @@ def get_filings_for_quarters(year_and_quarters: YearAndQuarters,
         _, final_index_table = fetch_filing_index(year_and_quarter=year_and_quarters[0],
                                                   index=index)
     else:
-        quarters_and_indexes = parallel(fetch_filing_index,
-                                        items=year_and_quarters,
-                                        index=index,
-                                        threadpool=True,
-                                        progress=True
-                                        )
+        quarters_and_indexes = parallel_thread_map(
+            lambda yq: fetch_filing_index(year_and_quarter=yq, index=index),
+            year_and_quarters
+        )
         quarter_and_indexes_sorted = sorted(quarters_and_indexes, key=lambda d: d[0])
         index_tables = [fd[1] for fd in quarter_and_indexes_sorted]
         final_index_table: pa.Table = pa.concat_tables(index_tables, mode="default")
@@ -434,6 +453,11 @@ class Filings:
         self.data_pager = DataPager(self.data)
         # This keeps track of where the index should start in case this is just a page in the Filings
         self._original_state = original_state or PagingState(0, len(self.data))
+        self._hash = None
+
+    @property
+    def docs(self):
+        return Docs(self)
 
     def to_pandas(self, *columns) -> pd.DataFrame:
         """Return the filing index as a python dataframe"""
@@ -447,6 +471,21 @@ class Filings:
     def save(self, location: str):
         """Save the filing index as parquet"""
         self.save_parquet(location)
+        
+    def download(self, data_directory: Optional[str] = None):
+        """
+        Download the filings based on the accession numbers in this Filings object.
+        
+        This is a convenience method that calls `download_filings` with this object
+        as the `filings` parameter.
+        
+        Args:
+            data_directory: Directory to save the downloaded files. Defaults to the Edgar data directory.
+        """
+        from edgar.storage import download_filings
+        download_filings(data_directory=data_directory, 
+                         overwrite_existing=True,
+                         filings=self)
 
     def get_filing_at(self, item: int):
         """Get the filing at the specified index"""
@@ -492,7 +531,7 @@ class Filings:
                cik: Union[IntString, List[IntString]] = None,
                exchange: Union[str, List[str], Exchange, List[Exchange]] = None,
                ticker: Union[str, List[str]] = None,
-               accession_number: Union[str, List[str]] = None) -> Optional['Filings']:
+               accession_number: Union[str, List[str]] = None) -> 'Filings':
         """
         Get some filings
 
@@ -543,7 +582,7 @@ class Filings:
                 filing_index = filter_by_date(filing_index, filing_date, 'filing_date')
             except InvalidDateException as e:
                 log.error(e)
-                return None
+                return Filings(_empty_filing_index())
 
         # Filter by cik
         if cik:
@@ -602,7 +641,7 @@ class Filings:
         """Show the next page"""
         data_page = self.data_pager.next()
         if data_page is None:
-            log.warning("End of data .. use prev() \u2190 ")
+            log.warning("End of data .. use previous() \u2190 ")
             return None
         start_index, _ = self.data_pager._current_range
         filings_state = PagingState(page_start=start_index, num_records=len(self))
@@ -621,10 +660,6 @@ class Filings:
         filings_state = PagingState(page_start=start_index, num_records=len(self))
         return Filings(data_page, original_state=filings_state)
 
-    def prev(self):
-        """Alias for self.previous()"""
-        return self.previous()
-
     def _get_by_accession_number(self, accession_number: str):
         mask = pc.equal(self.data['accession_number'], accession_number)
         idx = mask.index(True).as_py()
@@ -660,7 +695,7 @@ class Filings:
 
     def find(self,
              company_search_str: str):
-        from edgar.entities import find_company
+        from edgar.entity import find_company
 
         # Search for the company
         search_results = find_company(company_search_str)
@@ -704,6 +739,46 @@ class Filings:
         else:
             return range(*self.data_pager._current_range)
 
+    def __eq__(self, other):
+        # Check if other is Filings or subclass of Filings
+        if not isinstance(other, self.__class__) and not issubclass(other.__class__, self.__class__):
+            return False
+
+        if len(self) != len(other):
+            return False
+
+        if self.start_date != other.start_date or self.end_date != other.end_date:
+            return False
+
+        # Handle empty tables
+        if len(self) == 0:
+            return True  # Two empty tables with same dates are equal
+
+        # Compare just accession_number columns
+        return self.data['accession_number'].equals(other.data['accession_number'])
+
+
+    def __hash__(self):
+        if self._hash is None:
+            # Base hash components
+            hash_components = [self.__class__.__name__, len(self), self.start_date, self.end_date]
+
+            # Only add accession numbers if table is not empty
+            if len(self) > 0:
+                # Handle different table sizes appropriately
+                if len(self) == 1:
+                    hash_components.append(self.data['accession_number'][0].as_py())
+                elif len(self) == 2:
+                    hash_components.append(self.data['accession_number'][0].as_py())
+                    hash_components.append(self.data['accession_number'][1].as_py())
+                else:
+                    hash_components.append(self.data['accession_number'][0].as_py())
+                    hash_components.append(self.data['accession_number'][len(self) // 2].as_py())
+                    hash_components.append(self.data['accession_number'][len(self) - 1].as_py())
+
+            self._hash = hash(tuple(hash_components))
+        return self._hash
+
     def __rich__(self) -> Panel:
         # Create table with appropriate columns and styling
         table = Table(
@@ -718,12 +793,12 @@ class Filings:
 
         # Add columns with specific styling and alignment
         table.add_column("#", style="dim", justify="right")
-        table.add_column("Form", width=7)
+        table.add_column("Form", width=10)
         table.add_column("CIK", style="dim", width=10, justify="right")
         table.add_column("Ticker", width=6, style="yellow")
         table.add_column("Company", style="bold green", width=38, no_wrap=True)
         table.add_column("Filing Date", width=11)
-        table.add_column("Accession Number", style="dim", width=20)
+        table.add_column("Accession Number", width=20)
 
         # Get current page from data pager
         current_page = self.data_pager.current()
@@ -743,7 +818,7 @@ class Filings:
                 ticker,
                 current_page['company'][i].as_py(),
                 str(current_page['filing_date'][i].as_py()),
-                current_page['accession_number'][i].as_py()
+                accession_number_text(current_page['accession_number'][i].as_py())
             ]
             table.add_row(*row)
 
@@ -779,7 +854,8 @@ class Filings:
             Group(*elements),
             title="SEC Filings",
             subtitle=subtitle,
-            border_style="bold grey54"
+            border_style="bold grey54",
+            expand=False
         )
 
     def __repr__(self):
@@ -836,7 +912,7 @@ def get_filings(year: Optional[Years] = None,
                 amendments: bool = True,
                 filing_date: Optional[str] = None,
                 index="form",
-                priority_forms: Optional[List[str]] = None) -> Optional[Filings]:
+                priority_sorted_forms: Optional[List[str]] = None) -> Optional[Filings]:
     """
     Downloads the filing index for a given year or list of years, and a quarter or list of quarters.
 
@@ -873,19 +949,27 @@ def get_filings(year: Optional[Years] = None,
     :param filing_date The filing date to filter by in YYYY-MM-DD format
                 e.g. filing_date="2022-01-17" or filing_date="2022-01-17:2022-02-28"
     :param index The index type - "form" or "company" or "xbrl"
+    :param priority_sorted_forms: A list of forms to sort by priority. This presents these forms first for each day.
     :return:
     """
-    # Get the year or default to the current year
-    using_default_year = False
+    # Check if defaults were used
+    defaults_used = (year is None and
+                     quarter is None and
+                     form is None and
+                     amendments is True and
+                     filing_date is None and
+                     index == "form" and
+                     priority_sorted_forms is None)
     if filing_date:
         if not is_valid_filing_date(filing_date):
             log.warning("""Provide a valid filing date in the format YYYY-MM-DD or YYYY-MM-DD:YYYY-MM-DD""")
             return None
         year_and_quarters = filing_date_to_year_quarters(filing_date)
     elif not year:
+        # If no year specified, take the current year and quarter. (We need the quarter later)
         year, quarter = current_year_and_quarter()
-        year_and_quarters: YearAndQuarters = expand_quarters(year, quarter)
-        using_default_year = True
+        # Expand quarters for the year to date so use expand_quarters(year, quarter=None)
+        year_and_quarters: YearAndQuarters = expand_quarters(year, quarter=None)
     else:
         year_and_quarters: YearAndQuarters = expand_quarters(year, quarter)
 
@@ -907,16 +991,18 @@ def get_filings(year: Optional[Years] = None,
         filings = filings.filter(form=form, amendments=amendments, filing_date=filing_date)
 
     if not filings:
-        if using_default_year:
+        if defaults_used:
             # Ensure at least some data is returned
             previous_quarter = [get_previous_quarter(year, quarter)]
             filing_index = get_filings_for_quarters(previous_quarter, index=index)
-            sorted_filing_index = sort_filings_by_priority(filings.data, priority_forms)
+            filings = Filings(filing_index)
+            sorted_filing_index = sort_filings_by_priority(filings.data, priority_sorted_forms)
             return Filings(sorted_filing_index)
-        return None
+        # Return an empty filings object
+        return Filings(_empty_filing_index())
 
     # Sort the filings using the separate sort function
-    sorted_filing_index = sort_filings_by_priority(filings.data, priority_forms)
+    sorted_filing_index = sort_filings_by_priority(filings.data, priority_sorted_forms)
 
     return Filings(sorted_filing_index)
 
@@ -1007,7 +1093,7 @@ def get_current_filings(form: str = '',
     :return: The current filings from the SEC
     """
     owner = owner if owner in ['include', 'exclude', 'only'] else 'include'
-    page_size = page_size if page_size in [10, 20, 40, 80, 100] else 40
+    page_size = page_size if page_size in [10, 20, 40, 80, 100] else 100
     start = 0
 
     entries = get_current_entries_on_page(count=page_size, start=start, form=form, owner=owner)
@@ -1016,6 +1102,64 @@ def get_current_filings(form: str = '',
     return CurrentFilings(filing_index=pa.Table.from_pylist(entries), owner=owner, form=form, page_size=page_size)
 
 
+def iter_current_filings_pages(form: str = '',
+                              owner: str = 'include', 
+                              page_size: int = 100):
+    """
+    Iterator that yields CurrentFilings pages until exhausted.
+    
+    Args:
+        form: Form type to filter by (e.g., "10-K", "8-K")
+        owner: Owner filter ('include', 'exclude', 'only')
+        page_size: Number of filings per page (10, 20, 40, 80, 100)
+    
+    Yields:
+        CurrentFilings: Each page of current filings until no more pages
+        
+    Example:
+        >>> for page in iter_current_filings_pages(form="10-K"):
+        ...     print(f"Processing {len(page)} filings")
+        ...     # Process each page
+    """
+    current_page = get_current_filings(form=form, owner=owner, page_size=page_size)
+    
+    while current_page is not None:
+        yield current_page
+        current_page = current_page.next()
+
+
+def get_all_current_filings(form: str = '', 
+                           owner: str = 'include', 
+                           page_size: int = 100) -> Filings:
+    """
+    Get ALL current filings by iterating through all pages.
+    
+    Args:
+        form: Form type to filter by (e.g., "10-K", "8-K")
+        owner: Owner filter ('include', 'exclude', 'only')
+        page_size: Number of filings per page (10, 20, 40, 80, 100)
+    
+    Returns:
+        Filings: A regular Filings object containing all current filings
+        
+    Example:
+        >>> all_filings = get_all_current_filings(form="10-K")
+        >>> print(f"Found {len(all_filings)} total current 10-K filings")
+    """
+    all_entries = []
+    
+    for page in iter_current_filings_pages(form=form, owner=owner, page_size=page_size):
+        # Convert PyArrow table to list and extend
+        page_entries = page.data.to_pylist()
+        all_entries.extend(page_entries)
+    
+    if not all_entries:
+        return Filings(_empty_filing_index())
+    
+    # Return as regular Filings object (not CurrentFilings)
+    return Filings(pa.Table.from_pylist(all_entries))
+
+
 class CurrentFilings(Filings):
     """
     This version of the Filings class is used to get the current filings from the SEC
@@ -1039,7 +1183,7 @@ class CurrentFilings(Filings):
         if len(self.data) < self._page_size:
             return None
         start = self._start + len(self.data)
-        next_entries = get_current_entries_on_page(start=start, count=self._page_size, form=self.form)
+        next_entries = get_current_entries_on_page(start=start-1, count=self._page_size, form=self.form)
         if next_entries:
             # Copy the values to this Filings object and return it
             self.data = pa.Table.from_pylist(next_entries)
@@ -1244,6 +1388,11 @@ class Filing:
         self.filing_date = filing_date
         self.accession_no = accession_no
         self._filing_homepage = None
+        self._sgml = None
+
+    @property
+    def docs(self):
+        return Docs(self)
 
     @property
     def accession_number(self):
@@ -1257,6 +1406,13 @@ class Filing:
         document = self.sgml().attachments.primary_html_document
         # If the document is not in the SGML then we have to go to the homepage
         if document:
+            if document.extension == '.paper':
+                # If the document is a paper filing, we return the scanned document if it exists
+                attachments = self.homepage.attachments
+                scanned_documents = attachments.query("document == 'scanned.pdf'")
+                if len(scanned_documents) > 0:
+                    return scanned_documents.get_by_index(0)
+                return self.homepage.primary_html_document
             return document
         return self.homepage.primary_html_document
 
@@ -1275,7 +1431,7 @@ class Filing:
         """
         Get the period of report for the filing
         """
-        return self.homepage.period_of_report
+        return self.sgml().period_of_report
 
     @property
     def attachments(self):
@@ -1293,21 +1449,30 @@ class Filing:
         """Returns the html contents of the primary document if it is html"""
         sgml = self.sgml()
         html = sgml.html()
-        if html and not html.startswith("<?xml"):
-            # skip PDF (for now)
-            if html.endswith("</PDF>"):
+        if not html:
+            document:Attachment = self.homepage.primary_html_document
+            if document.empty or document.is_binary():
                 return None
-            if has_html_content(html):
-                return html
+            return self.homepage.primary_html_document.download()
+        if html.endswith("</PDF>"):
             return None
-        # If the html document is not in the SGML then we have to go to the homepage
-        html = self.homepage.primary_html_document.download()
+        if html.startswith("<?xml"):
+            if self.form in ['3','3/A', '4', '4/A', '5', '5/A']:
+                from edgar.ownership import Ownership
+                ownership:Ownership = self.obj()
+                html = ownership.to_html()
+            else:
+                html = self.homepage.primary_html_document.download()
         if isinstance(html, bytes):
             try:
                 return html.decode("utf-8")
             except UnicodeDecodeError:
                 return None
-        return html
+        if is_probably_html(html):
+            return html
+        else:
+            html = html.replace("<PAGE>", "")
+            return f"<html><body><div>{html}</div></body></html>"
 
     @lru_cache(maxsize=4)
     def xml(self) -> Optional[str]:
@@ -1319,7 +1484,7 @@ class Filing:
     def text(self) -> str:
         """Convert the html of the main filing document to text"""
         html_content = self.html()
-        if html_content and has_html_content(html_content):
+        if html_content and is_probably_html(html_content):
             document = Document.parse(html_content)
             return rich_to_text(document)
         else:
@@ -1368,12 +1533,15 @@ class Filing:
         else:
             print(self.text())
 
-    def xbrl(self) -> Optional[Union[XBRLData, XBRLInstance]]:
+    def xbrl(self) -> Optional[XBRL]:
         """
         Get the XBRL document for the filing, parsed and as a FilingXbrl object
         :return: Get the XBRL document for the filing, parsed and as a FilingXbrl object, or None
         """
-        return get_xbrl_object(self)
+        try:
+            return XBRL.from_filing(self)
+        except XBRLFilingWithNoXbrlData:
+            return None
 
     def serve(self, port: int = 8000) -> AttachmentServer:
         """Serve the filings on a local server
@@ -1407,16 +1575,46 @@ class Filing:
     def filing_directory(self) -> FilingDirectory:
         return FilingDirectory.load(self.base_dir)
 
-    @lru_cache()
+    def _local_path(self) -> Path:
+        """
+        Get the local path for the filing
+        """
+        return local_filing_path(str(self.filing_date), self.accession_no)
+
+    @classmethod
+    def from_sgml(cls, source: Union[str, Path]):
+        """
+        Read the filing from the SGML string
+        """
+        filing_sgml = FilingSGML.from_source(source)
+        filers = filing_sgml.header.filers
+        if filers and len(filers) > 0:
+             company = filers[0].company_information.name if filers[0].company_information else ""
+        else:
+            company = ""
+
+        filing = cls(cik=filing_sgml.cik,
+                   accession_no=filing_sgml.accession_number,
+                   form=filing_sgml.form,
+                   company=company,
+                   filing_date=filing_sgml.filing_date)
+        filing._sgml = filing_sgml
+        return filing
+
     def sgml(self) -> FilingSGML:
         """
         Read the filing from the local storage path if it exists
         """
+        if self._sgml:
+            return self._sgml
         if is_using_local_storage():
             local_path = local_filing_path(str(self.filing_date), self.accession_no)
             if local_path.exists():
-                return FilingSGML.from_source(local_path)
-        return FilingSGML.from_filing(self)
+                self._sgml = FilingSGML.from_source(local_path)
+
+        if self._sgml is None:
+            self._sgml = FilingSGML.from_filing(self)
+        return self._sgml
 
     @cached_property
     def reports(self)  -> Optional[Reports]:
@@ -1475,18 +1673,8 @@ class Filing:
     @property
     @lru_cache(maxsize=1)
     def header(self):
-        if is_using_local_storage():
-            _sgml = self.sgml()
-            if _sgml:
-                return _sgml.header
-
-        sec_header_content = download_text_between_tags(self.text_url, "SEC-HEADER")
-        try:
-            return FilingHeader.parse_from_sgml_text(sec_header_content)
-        except Exception:
-            # Try again with preprocessing - likely with a filing from the 1990's
-            return FilingHeader.parse_from_sgml_text(sec_header_content, preprocess=True)
-
+        _sgml = self.sgml()
+        return _sgml.header
 
 
     def data_object(self):
@@ -1573,8 +1761,8 @@ class Filing:
     def get_entity(self):
         """Get the company to which this filing belongs"""
         "Get the company for cik. Cache for performance"
-        from edgar.entities import CompanyData
-        return CompanyData.for_cik(self.cik)
+        from edgar.entity import Company
+        return Company(self.cik)
 
     @lru_cache(maxsize=1)
     def as_company_filing(self):
@@ -1586,6 +1774,7 @@ class Filing:
         filings = company.get_filings(accession_number=self.accession_no)
         if filings and not filings.empty:
             return filings[0]
+        return None
 
     @lru_cache(maxsize=1)
     def related_filings(self):
@@ -1603,7 +1792,7 @@ class Filing:
             if is_using_local_storage():
                 # In this case the local storage is missing the filing so we have to download it
                 log.warning(f"Filing {self.accession_no} not found in local storage. Downloading from SEC ...")
-                from edgar.entities import download_entity_submissions_from_sec, parse_entity_submissions
+                from edgar.entity import download_entity_submissions_from_sec, parse_entity_submissions
                 submissions_json = download_entity_submissions_from_sec(self.cik)
                 c_from_sec = parse_entity_submissions(submissions_json)
                 filings = c_from_sec.get_filings(accession_number=self.accession_no)
@@ -1654,7 +1843,7 @@ class Filing:
         └──────────────────────┴──────┴────────────┴────────────────────┴─────────┘
         :return: a rich table version of this filing
         """
-        ticker = find_ticker(self.cik)
+        ticker = find_ticker_safe(self.cik)
         ticker = f"{ticker}" if ticker else ""
 
         # The title of the panel
@@ -1672,7 +1861,7 @@ class Filing:
         filing_info_table = Table("Accession Number", "Filing Date", "Period of Report", "Documents",
                                   header_style="dim",
                                   box=box.SIMPLE_HEAD)
-        filing_info_table.add_row(Text(self.accession_no, "bold deep_sky_blue1"),
+        filing_info_table.add_row(accession_number_text(self.accession_no),
                                   Text(str(self.filing_date), "bold"),
                                   Text(self.period_of_report or "-", "bold"),
                                   f"{len(attachments)}")
@@ -1681,7 +1870,7 @@ class Filing:
             title=title,
             subtitle=subtitle,
             box=box.ROUNDED,
-            height=10,
+            height=8,
             expand=False
         )
 
@@ -1745,7 +1934,7 @@ def get_filing_by_accession(accession_number: str, year: int):
     return None
 
 
-def get_by_accession_number(accession_number: str, show_progress: bool = True):
+def get_by_accession_number(accession_number: str, show_progress: bool = False):
     """Wrapper that handles progress display and current time logic"""
     year = int("19" + accession_number[11:13]) if accession_number[11] == '9' else int("20" + accession_number[11:13])
 
